// Main include file: bit masks, timer rates, data structures, prototypes
// Copyright 2003, Ilya Mironov & Dmitry Teytelman

#ifndef _BUTTONS_H_
  #define _BUTTONS_H_

  // We handle 12 pushbuttons
  #define NUMBUTTONS	12

  #define BUTTONMASK	0xfff

  // Serial port baud rate config
  // System oscillator in Hz
  #define FOSC		22118400
  #define BAUDRATE	57600
  // Compute timer reload value
  #define TH1INIT	(unsigned char)(256-FOSC/16/12/BAUDRATE)
  
  // Timer 0 setup - interrupt rate in us
  #define TIMER0RATE	100
  #define TH0INIT	(unsigned char)(256-FOSC*TIMER0RATE/12/1000000)
  #define TICKS_PER_SEC	(1000000/TIMER0RATE)

  // LED timer bit shifts
  // Fast blinking - 10 bit shift, 0.2 sec period
  #define LED_C_FAST	10
  // Slow blinking - 12 bit shift, 0.8 sec period
  #define LED_C_SLOW	12

  // System state bits
  enum state
  {
    STATE_STOP,
    STATE_RUN
  };
  
  typedef struct _event
  {
    unsigned long int timer;
  } event;

  void hwinit(void);

  // Serial interface functions
  void ser_handler (void) interrupt 4 using 2;
  void ser_putc(char c);
  void ser_puts(char *str);
  void send_msg(unsigned int evtType, unsigned int mask, unsigned long timer);

  // Timer functions
  void TimerIrqHandler(void) interrupt 1 using 1;
  unsigned long RdTimer(void);
  void ClrTimer(void);
  
  // Printing functions
  void print_hex(unsigned int val);
  void print_hexl(unsigned long int val);

#endif
