/*
	Serial communication functions used by mcr.cpp to talk to MINI/MAX-51C
*/
// Copyright 2003, Ilya Mironov & Dmitry Teytelman

#include "../stdafx.h"
#include <stdio.h>
#include <time.h>
#include <string.h>
#include "serial.h"

bool bKeyboard = false;

HANDLE SerialInit(TCHAR *ComPortName, int BaudRate) 
{
	HANDLE hCom;
    BOOL bPortReady;
    DCB	dcb;
    COMMTIMEOUTS CommTimeouts;
	
	hCom = CreateFile(ComPortName, 
		GENERIC_READ | GENERIC_WRITE,
		0, // exclusive access
		NULL, // no security
		OPEN_EXISTING,
		0, // no overlapped I/O
		NULL); // null template 

	// set buffer sizes
	bPortReady = SetupComm(hCom, SER_IBUFSIZE, SER_OBUFSIZE); 

	bPortReady = GetCommState(hCom, &dcb);
	dcb.BaudRate = BaudRate;
	dcb.ByteSize = 8;
	dcb.Parity = NOPARITY;
//	dcb.Parity = EVENPARITY;
	dcb.StopBits = ONESTOPBIT;
	dcb.fAbortOnError = TRUE;

	// set XON/XOFF
	dcb.fOutX = FALSE;					// XON/XOFF off for transmit
	dcb.fInX	= FALSE;					// XON/XOFF off for receive
	// set RTSCTS
	dcb.fOutxCtsFlow = FALSE;					// turn off CTS flow control
	dcb.fRtsControl = RTS_CONTROL_DISABLE;		// turn off RTS
	// set DSRDTR
	dcb.fOutxDsrFlow = FALSE;					// turn on DSR flow control
	dcb.fDtrControl = DTR_CONTROL_ENABLE;	// 
//	dcb.fDtrControl = DTR_CONTROL_DISABLE;	// 
//	dcb.fDtrControl = DTR_CONTROL_HANDSHAKE;	// 

	bPortReady = SetCommState(hCom, &dcb);

	// Communication timeouts are optional

	bPortReady = GetCommTimeouts (hCom, &CommTimeouts);

	CommTimeouts.ReadIntervalTimeout = 0;
	CommTimeouts.ReadTotalTimeoutConstant = 200;
	CommTimeouts.ReadTotalTimeoutMultiplier = 0;
	CommTimeouts.WriteTotalTimeoutConstant = 5000;
	CommTimeouts.WriteTotalTimeoutMultiplier = 1000;

	bPortReady = SetCommTimeouts (hCom, &CommTimeouts);

	return hCom;
}

char SerialGetc(HANDLE *hCom)
{
	char rxchar;
	BOOL	bReadRC;
	static	DWORD	iBytesRead;

	bReadRC = ReadFile(*hCom, &rxchar, 1, &iBytesRead, NULL);

	return rxchar;
}

void SerialPutc(HANDLE *hCom, char txchar)
{
	BOOL	bWriteRC;
	static	DWORD	iBytesWritten;
	
	bWriteRC = WriteFile(*hCom, &txchar, 1, &iBytesWritten,NULL);
}

char* SerialGets(HANDLE *hCom)
{
	static char rxstring[256];
	char c;
	int pos = 0;

	while(pos <= 255) {
		c = SerialGetc(hCom);
		if(c == '\r') break;
		rxstring[pos++] = c;
	}
	rxstring[pos] = 0;

	return	rxstring;
}

void SerialPuts(HANDLE *hCom, char *txstring)
{
	BOOL	bWriteRC;
	static	DWORD	iBytesWritten;

	bWriteRC = WriteFile(*hCom, txstring, strlen(txstring), &iBytesWritten,NULL);

}

/*
   Check for data in the serial input buffer - returns number of
   bytes waiting
*/
int SerialInBuf(HANDLE *hCom)
{
	DWORD foo;
	COMSTAT stat;

	if(bKeyboard)
		return 0;

	ClearCommError(*hCom, &foo, &stat);

	return stat.cbInQue;
}

/*
   Read count bytes from the serial port. If timeout is exceeded, fewer
   bytes will be read. Return value indicates how many bytes were
   placed in buf.
*/
unsigned long SerialRead(HANDLE *hCom, void *buf, int count)
{
	unsigned long rc;

	ReadFile(*hCom, buf, count, &rc, NULL);
	return rc;
}
